@@ -8,7 +8,10 @@ Examples::
 
     $ python runtests.py
     $ python runtests.py -s {SAMPLE_SUBMODULE}
+    $ # Run a standalone test function:
     $ python runtests.py -t {SAMPLE_TEST}
+    $ # Run a test defined as a method of a TestXXX class:
+    $ python runtests.py -t {SAMPLE_TEST2}
     $ python runtests.py --ipython
     $ python runtests.py --python somescript.py
     $ python runtests.py --bench
@@ -28,6 +31,12 @@ Generate C code coverage listing under build/lcov/:
     $ python runtests.py --gcov [...other args...]
     $ python runtests.py --lcov-html
 
+Run lint checks.
+Provide target branch name or `uncommitted` to check before committing:
+
+    $ python runtests.py --lint main
+    $ python runtests.py --lint uncommitted
+
 """
 #
 # This is a generic test runner script for projects using NumPy's test
@@ -37,6 +46,7 @@ Generate C code coverage listing under build/lcov/:
 PROJECT_MODULE = "numpy"
 PROJECT_ROOT_FILES = ['numpy', 'LICENSE.txt', 'setup.py']
 SAMPLE_TEST = "numpy/linalg/tests/test_linalg.py::test_byteorder_check"
+SAMPLE_TEST2 = "numpy/core/tests/test_memmap.py::TestMemmap::test_open_with_filename"
 SAMPLE_SUBMODULE = "linalg"
 
 EXTRA_PATH = ['/usr/lib/ccache', '/usr/lib/f90cache',
@@ -52,7 +62,7 @@ else:
 
 
 import sys
-import os
+import os, glob
 
 # In case we are run from the source directory, we don't want to import the
 # project from there:
@@ -68,36 +78,44 @@ ROOT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__)))
 def main(argv):
     parser = ArgumentParser(usage=__doc__.lstrip())
     parser.add_argument("--verbose", "-v", action="count", default=1,
-                        help="more verbosity")
+                        help="Add one verbosity level to pytest. Default is 0")
     parser.add_argument("--debug-info", action="store_true",
-                        help=("add --verbose-cfg to build_src to show compiler "
-                              "configuration output while creating "
+                        help=("Add --verbose-cfg to build_src to show "
+                              "compiler configuration output while creating "
                               "_numpyconfig.h and config.h"))
     parser.add_argument("--no-build", "-n", action="store_true", default=False,
-                        help="do not build the project (use system installed version)")
-    parser.add_argument("--build-only", "-b", action="store_true", default=False,
-                        help="just build, do not run any tests")
+                        help="Do not build the project (use system installed "
+                             "version)")
+    parser.add_argument("--build-only", "-b", action="store_true",
+                        default=False, help="Just build, do not run any tests")
     parser.add_argument("--doctests", action="store_true", default=False,
                         help="Run doctests in module")
     parser.add_argument("--refguide-check", action="store_true", default=False,
-                        help="Run refguide (doctest) check (do not run regular tests.)")
+                        help="Run refguide (doctest) check (do not run "
+                             "regular tests.)")
     parser.add_argument("--coverage", action="store_true", default=False,
-                        help=("report coverage of project code. HTML output goes "
-                              "under build/coverage"))
+                        help=("Report coverage of project code. HTML output "
+                              "goes under build/coverage"))
+    parser.add_argument("--lint", default=None,
+                        help="'<Target Branch>' or 'uncommitted', passed to "
+                             "tools/linter.py [--branch BRANCH] "
+                             "[--uncommitted]")
     parser.add_argument("--durations", action="store", default=-1, type=int,
-                        help=("Time N slowest tests, time all if 0, time none if < 0"))
+                        help=("Time N slowest tests, time all if 0, time none "
+                              "if < 0"))
     parser.add_argument("--gcov", action="store_true", default=False,
-                        help=("enable C code coverage via gcov (requires GCC). "
-                              "gcov output goes to build/**/*.gc*"))
+                        help=("Enable C code coverage via gcov (requires "
+                              "GCC). gcov output goes to build/**/*.gc*"))
     parser.add_argument("--lcov-html", action="store_true", default=False,
-                        help=("produce HTML for C code coverage information "
+                        help=("Produce HTML for C code coverage information "
                               "from a previous run with --gcov. "
                               "HTML output goes to build/lcov/"))
     parser.add_argument("--mode", "-m", default="fast",
                         help="'fast', 'full', or something that could be "
                              "passed to nosetests -A [default: fast]")
     parser.add_argument("--submodule", "-s", default=None,
-                        help="Submodule whose tests to run (cluster, constants, ...)")
+                        help="Submodule whose tests to run (cluster, "
+                             "constants, ...)")
     parser.add_argument("--pythonpath", "-p", default=None,
                         help="Paths to prepend to PYTHONPATH")
     parser.add_argument("--tests", "-t", action='append',
@@ -108,12 +126,26 @@ def main(argv):
                         help="Start IPython shell with PYTHONPATH set")
     parser.add_argument("--shell", action="store_true",
                         help="Start Unix shell with PYTHONPATH set")
+    parser.add_argument("--mypy", action="store_true",
+                        help="Run mypy on files with NumPy on the MYPYPATH")
     parser.add_argument("--debug", "-g", action="store_true",
                         help="Debug build")
     parser.add_argument("--parallel", "-j", type=int, default=0,
                         help="Number of parallel jobs during build")
     parser.add_argument("--warn-error", action="store_true",
-                        help="Set -Werror to convert all compiler warnings to errors")
+                        help="Set -Werror to convert all compiler warnings to "
+                             "errors")
+    parser.add_argument("--cpu-baseline", default=None,
+                        help="Specify a list of enabled baseline CPU "
+                             "optimizations"),
+    parser.add_argument("--cpu-dispatch", default=None,
+                        help="Specify a list of dispatched CPU optimizations"),
+    parser.add_argument("--disable-optimization", action="store_true",
+                        help="Disable CPU optimized code (dispatch, simd, "
+                             "fast, ...)"),
+    parser.add_argument("--simd-test", default=None,
+                        help="Specify a list of CPU optimizations to be "
+                             "tested against NumPy SIMD interface"),
     parser.add_argument("--show-build-log", action="store_true",
                         help="Show build output rather than using a log file")
     parser.add_argument("--bench", action="store_true",
@@ -125,7 +157,8 @@ def main(argv):
                               "COMMIT. Note that you need to commit your "
                               "changes first!"))
     parser.add_argument("args", metavar="ARGS", default=[], nargs=REMAINDER,
-                        help="Arguments to pass to Nose, asv, Python or shell")
+                        help="Arguments to pass to pytest, asv, mypy, Python "
+                             "or shell")
     args = parser.parse_args(argv)
 
     if args.durations < 0:
@@ -151,6 +184,9 @@ def main(argv):
         print("*** Benchmarks should not be run against debug "
               "version; remove -g flag ***")
 
+    if args.lint:
+        check_lint(args.lint)
+
     if not args.no_build:
         # we need the noarch path in case the package is pure python.
         site_dir, site_dir_noarch = build_project(args)
@@ -196,7 +232,7 @@ def main(argv):
         import warnings; warnings.filterwarnings("always")
         import IPython
         import numpy as np
-        IPython.embed(user_ns={"np": np})
+        IPython.embed(colors='neutral', user_ns={"np": np})
         sys.exit(0)
 
     if args.shell:
@@ -205,6 +241,36 @@ def main(argv):
         subprocess.call([shell] + extra_argv)
         sys.exit(0)
 
+    if args.mypy:
+        try:
+            import mypy.api
+        except ImportError:
+            raise RuntimeError(
+                "Mypy not found. Please install it by running "
+                "pip install -r test_requirements.txt from the repo root"
+            )
+
+        os.environ['MYPYPATH'] = site_dir
+        # By default mypy won't color the output since it isn't being
+        # invoked from a tty.
+        os.environ['MYPY_FORCE_COLOR'] = '1'
+
+        config = os.path.join(
+            site_dir,
+            "numpy",
+            "typing",
+            "tests",
+            "data",
+            "mypy.ini",
+        )
+
+        report, errors, status = mypy.api.run(
+            ['--config-file', config] + args.args
+        )
+        print(report, end='')
+        print(errors, end='', file=sys.stderr)
+        sys.exit(status)
+
     if args.coverage:
         dst_dir = os.path.join(ROOT_DIR, 'build', 'coverage')
         fn = os.path.join(dst_dir, 'coverage_html.js')
@@ -272,8 +338,16 @@ def main(argv):
             out = subprocess.check_output(['git', 'rev-parse', commit_a])
             commit_a = out.strip().decode('ascii')
 
+            # generate config file with the required build options
+            asv_cfpath = [
+                '--config', asv_compare_config(
+                    os.path.join(ROOT_DIR, 'benchmarks'), args,
+                    # to clear the cache if the user changed build options
+                    (commit_a, commit_b)
+                )
+            ]
             cmd = ['asv', 'continuous', '-e', '-f', '1.05',
-                   commit_a, commit_b] + bench_args
+                   commit_a, commit_b] + asv_cfpath + bench_args
             ret = subprocess.call(cmd, cwd=os.path.join(ROOT_DIR, 'benchmarks'))
             sys.exit(ret)
 
@@ -323,7 +397,6 @@ def main(argv):
     else:
         sys.exit(1)
 
-
 def build_project(args):
     """
     Build a dev version of the project.
@@ -335,7 +408,7 @@ def build_project(args):
 
     """
 
-    import distutils.sysconfig
+    import sysconfig
 
     root_ok = [os.path.exists(os.path.join(ROOT_DIR, fn))
                for fn in PROJECT_ROOT_FILES]
@@ -351,7 +424,7 @@ def build_project(args):
 
     # Always use ccache, if installed
     env['PATH'] = os.pathsep.join(EXTRA_PATH + env.get('PATH', '').split(os.pathsep))
-    cvars = distutils.sysconfig.get_config_vars()
+    cvars = sysconfig.get_config_vars()
     compiler = env.get('CC') or cvars.get('CC', '')
     if 'gcc' in compiler:
         # Check that this isn't clang masquerading as gcc.
@@ -384,25 +457,48 @@ def build_project(args):
     cmd += ["build"]
     if args.parallel > 1:
         cmd += ["-j", str(args.parallel)]
-    if args.debug_info:
-        cmd += ["build_src", "--verbose-cfg"]
     if args.warn_error:
         cmd += ["--warn-error"]
+    if args.cpu_baseline:
+        cmd += ["--cpu-baseline", args.cpu_baseline]
+    if args.cpu_dispatch:
+        cmd += ["--cpu-dispatch", args.cpu_dispatch]
+    if args.disable_optimization:
+        cmd += ["--disable-optimization"]
+    if args.simd_test is not None:
+        cmd += ["--simd-test", args.simd_test]
+    if args.debug_info:
+        cmd += ["build_src", "--verbose-cfg"]
     # Install; avoid producing eggs so numpy can be imported from dst_dir.
     cmd += ['install', '--prefix=' + dst_dir,
             '--single-version-externally-managed',
             '--record=' + dst_dir + 'tmp_install_log.txt']
 
-    from distutils.sysconfig import get_python_lib
-    site_dir = get_python_lib(prefix=dst_dir, plat_specific=True)
-    site_dir_noarch = get_python_lib(prefix=dst_dir, plat_specific=False)
+    py_v_s = sysconfig.get_config_var('py_version_short')
+    platlibdir = getattr(sys, 'platlibdir', '')  # Python3.9+
+    site_dir_template = os.path.normpath(sysconfig.get_path(
+        'platlib', expand=False
+    ))
+    site_dir = site_dir_template.format(platbase=dst_dir,
+                                        py_version_short=py_v_s,
+                                        platlibdir=platlibdir,
+                                        base=dst_dir,
+                                        )
+    noarch_template = os.path.normpath(sysconfig.get_path(
+        'purelib', expand=False
+    ))
+    site_dir_noarch = noarch_template.format(base=dst_dir,
+                                             py_version_short=py_v_s,
+                                             platlibdir=platlibdir,
+                                             )
+
     # easy_install won't install to a path that Python by default cannot see
     # and isn't on the PYTHONPATH.  Plus, it has to exist.
     if not os.path.exists(site_dir):
         os.makedirs(site_dir)
     if not os.path.exists(site_dir_noarch):
         os.makedirs(site_dir_noarch)
-    env['PYTHONPATH'] = site_dir + ':' + site_dir_noarch
+    env['PYTHONPATH'] = site_dir + os.pathsep + site_dir_noarch
 
     log_filename = os.path.join(ROOT_DIR, 'build.log')
 
@@ -447,6 +543,99 @@ def build_project(args):
 
     return site_dir, site_dir_noarch
 
+def asv_compare_config(bench_path, args, h_commits):
+    """
+    Fill the required build options through custom variable
+    'numpy_build_options' and return the generated config path.
+    """
+    conf_path = os.path.join(bench_path, "asv_compare.conf.json.tpl")
+    nconf_path = os.path.join(bench_path, "_asv_compare.conf.json")
+
+    # add custom build
+    build = []
+    if args.parallel > 1:
+        build += ["-j", str(args.parallel)]
+    if args.cpu_baseline:
+        build += ["--cpu-baseline", args.cpu_baseline]
+    if args.cpu_dispatch:
+        build += ["--cpu-dispatch", args.cpu_dispatch]
+    if args.disable_optimization:
+        build += ["--disable-optimization"]
+
+    is_cached = asv_substitute_config(conf_path, nconf_path,
+        numpy_build_options = ' '.join([f'\\"{v}\\"' for v in build]),
+        numpy_global_options= ' '.join([f'--global-option=\\"{v}\\"' for v in ["build"] + build])
+    )
+    if not is_cached:
+        asv_clear_cache(bench_path, h_commits)
+    return nconf_path
+
+def asv_clear_cache(bench_path, h_commits, env_dir="env"):
+    """
+    Force ASV to clear the cache according to specified commit hashes.
+    """
+    # FIXME: only clear the cache from the current environment dir
+    asv_build_pattern = os.path.join(bench_path, env_dir, "*", "asv-build-cache")
+    for asv_build_cache in glob.glob(asv_build_pattern, recursive=True):
+        for c in h_commits:
+            try: shutil.rmtree(os.path.join(asv_build_cache, c))
+            except OSError: pass
+
+def asv_substitute_config(in_config, out_config, **custom_vars):
+    """
+    A workaround to allow substituting custom tokens within
+    ASV configuration file since there's no official way to add custom
+    variables(e.g. env vars).
+
+    Parameters
+    ----------
+    in_config : str
+        The path of ASV configuration file, e.g. '/path/to/asv.conf.json'
+    out_config : str
+        The path of generated configuration file,
+        e.g. '/path/to/asv_substituted.conf.json'.
+
+    The other keyword arguments represent the custom variables.
+
+    Returns
+    -------
+    True(is cached) if 'out_config' is already generated with
+    the same '**custom_vars' and updated with latest 'in_config',
+    False otherwise.
+
+    Examples
+    --------
+    See asv_compare_config().
+    """
+    assert in_config != out_config
+    assert len(custom_vars) > 0
+
+    def sdbm_hash(*factors):
+        chash = 0
+        for f in factors:
+            for char in str(f):
+                chash  = ord(char) + (chash << 6) + (chash << 16) - chash
+                chash &= 0xFFFFFFFF
+        return chash
+
+    vars_hash = sdbm_hash(custom_vars, os.path.getmtime(in_config))
+    try:
+        with open(out_config, "r") as wfd:
+            hash_line = wfd.readline().split('hash:')
+            if len(hash_line) > 1 and int(hash_line[1]) == vars_hash:
+                return True
+    except IOError:
+        pass
+
+    custom_vars = {f'{{{k}}}':v for k, v in custom_vars.items()}
+    with open(in_config, "r") as rfd, open(out_config, "w") as wfd:
+        wfd.write(f"// hash:{vars_hash}\n")
+        wfd.write("// This file is automatically generated by runtests.py\n")
+        for line in rfd:
+            for key, val in custom_vars.items():
+                line = line.replace(key, val)
+            wfd.write(line)
+    return False
 
 #
 # GCOV support
@@ -488,6 +677,24 @@ def lcov_generate():
     else:
         print("HTML output generated under build/lcov/")
 
+def check_lint(lint_args):
+    """
+    Adds ROOT_DIR to path and performs lint checks.
+    This functions exits the program with status code of lint check.
+    """
+    sys.path.append(ROOT_DIR)
+    try:
+        from tools.linter import DiffLinter
+    except ModuleNotFoundError as e:
+        print(f"Error: {e.msg}. "
+              "Install using linter_requirements.txt.")
+        sys.exit(1)
+
+    uncommitted = lint_args == "uncommitted"
+    branch = "main" if uncommitted else lint_args
+
+    DiffLinter(branch).run_lint(uncommitted)
+
 
 if __name__ == "__main__":
     main(argv=sys.argv[1:])
